<?php

namespace MDV\ProfileBackground\Service\ProfileBackground;

use XF;
use XF\App;
use XF\Entity\User;
use XF\Service\AbstractService;
use XF\Util\File;
use XF\Http\Upload;
use LogicException;
use Exception;

class ProfileBackground extends AbstractService 
{
    /**
     * @var User
    */
    protected $user;

    protected $fileName;

    protected $error = null;

    protected $throwErrors = true;

    protected $allowedTypes = [IMAGETYPE_GIF, IMAGETYPE_JPEG, IMAGETYPE_PNG];

    protected $sizeMap;

    public function __construct(App $app, User $user)
	{
		parent::__construct($app);
		$this->setUser($user);

        $this->sizeMap = $app->container('mdv_profileBackgroundSizeMap');
	}

    protected function setUser(User $user)
	{
		if ($user->user_id)
		{
			$this->user = $user;
		}
		else
		{
			throw new \LogicException("User must be saved");
		}
	}

	public function getError()
	{
		return $this->error;
	}

    public function setImage($fileName)
	{
		if (!$this->validateImageAsBackground($fileName, $error))
		{
			$this->error = $error;
			$this->fileName = null;
			return false;
		}

		$this->fileName = $fileName;
		return true;
	}

    public function setImageFromUpload(Upload $upload)
	{
		$upload->requireImage();

		if (!$upload->isValid($errors))
		{
			$this->error = reset($errors);
			return false;
		}

		return $this->setImage($upload->getTempFile());
	}

    public function validateImageAsBackground($fileName, &$error = null): bool
	{
		$error = null;

		if (!file_exists($fileName))
		{
			return $this->throwException(new \InvalidArgumentException("Invalid file '$fileName' passed to avatar service"));
		}
		if (!is_readable($fileName))
		{
			return $this->throwException(new \InvalidArgumentException("'$fileName' passed to background service is not readable"));
		}

		$imageInfo = filesize($fileName) ? @getimagesize($fileName) : false;
		if (!$imageInfo)
		{
			$error = \XF::phrase('provided_file_is_not_valid_image');
			return false;
		}

		$type = $imageInfo[2];
		if (!in_array($type, $this->allowedTypes))
		{
			$error = \XF::phrase('provided_file_is_not_valid_image');
			return false;
		}

		$width = $imageInfo[0];
		$height = $imageInfo[1];

		if (!$this->app->imageManager()->canResize($width, $height))
		{
			$error = XF::phrase('uploaded_image_is_too_big');
			return false;
		}

		$this->width = $width;
		$this->height = $height;
		$this->type = $type;

		return true;
	}

	public function updateBackground(): bool
	{
		if (!$this->fileName)
		{
			return $this->throwException(new LogicException("No source file for background set"));
		}
		if (!$this->user->exists())
		{
			return $this->throwException(new LogicException("User does not exist, cannot update background"));
		}

		$imageManager = $this->app->imageManager();

		$outputFiles = [];

		foreach ($this->sizeMap AS $size => $width)
		{
			$image = $imageManager->imageFromFile($this->fileName);
			if (!$image)
			{
				continue;
			}

			$image->resizeWidth($width, true);

			$newTempFile = File::getTempFile();
			if ($newTempFile && $image->save($newTempFile, null, 95))
			{
				$outputFiles[$size] = $newTempFile;
			}
			unset($image);
		}

		if (count($outputFiles) != count($this->sizeMap))
		{
			return $this->throwException(new \RuntimeException("Failed to save image to temporary file; image may be corrupt or check internal_data/data permissions"));
		}

		foreach ($outputFiles AS $code => $file)
		{
			$dataFile = $this->user->Profile->getAbstractedBackgroundPath($code);
			File::copyFileToAbstractedPath($file, $dataFile);
		}

		$profile = $this->user->Profile;
        $profile->set('mdv_background_date', XF::$time);
		$profile->save();

		return true;
	}

    public function deleteBackground(): bool
	{
		$this->deleteBackgroundFiles();

		$profile = $this->user->Profile;
		$profile->set('mdv_background_date', 0);
		$profile->save();

		return true;
	}

    public function deleteBackgroundForUserDelete(): bool
	{
		$this->deleteBackgroundFiles();

		return true;
	}

    protected function deleteBackgroundFiles()
	{
		if ($this->user->Profile->mdv_background_date)
		{
			foreach ($this->sizeMap AS $code => $size)
			{
				File::deleteFromAbstractedPath($this->user->Profile->getAbstractedBackgroundPath($code));
			}
		}
	}

    /**
	 * @param \Exception $error
	 *
	 * @return bool
	 * @throws \Exception
	 */
	protected function throwException(Exception $error)
	{
		if ($this->throwErrors)
		{
			throw $error;
		}
		else
		{
			return false;
		}
	}
}